@@ -1,16 +1,16 @@
 /* actions.c
  *
- * Copyright (C) 2002 Lutz Mller <lutz@users.sourceforge.net>
+ * Copyright  2002 Lutz Mller <lutz@users.sourceforge.net>
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
  * License as published by the Free Software Foundation; either
  * version 2 of the License, or (at your option) any later version.
  *
- * This library is distributed in the hope that it will be useful, 
- * but WITHOUT ANY WARRANTY; without even the implied warranty of 
+ * This library is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
- * Lesser General Public License for more details. 
+ * Lesser General Public License for more details.
  *
  * You should have received a copy of the GNU Lesser General Public
  * License along with this library; if not, write to the
@@ -18,8 +18,10 @@
  * Boston, MA 02111-1307, USA.
  */
 
-#include <config.h>
+#include "config.h"
 #include "actions.h"
+#include "i18n.h"
+#include "main.h"
 #include "version.h"
 
 #include <string.h>
@@ -33,8 +35,6 @@
 #include <gphoto2/gphoto2-port-log.h>
 #include <gphoto2/gphoto2-setting.h>
 
-#include "main.h"
-
 #ifdef HAVE_AA
 #  include "gphoto2-cmd-capture.h"
 #endif
@@ -43,27 +43,9 @@
 #  include <libexif/exif-data.h>
 #endif
 
-#ifdef ENABLE_NLS
-#  include <libintl.h>
-#  undef _
-#  define _(String) dgettext (PACKAGE, String)
-#  ifdef gettext_noop
-#    define N_(String) gettext_noop (String)
-#  else
-#    define N_(String) (String)
-#  endif
-#else
-#  define textdomain(String) (String)
-#  define gettext(String) (String)
-#  define dgettext(Domain,Message) (Message)
-#  define dcgettext(Domain,Message,Type) (Message)
-#  define bindtextdomain(Domain,Directory) (Domain)
-#  define _(String) (String)
-#  define N_(String) (String)
-#endif
-
 #define CR(result)       {int r=(result); if (r<0) return r;}
 #define CRU(result,file) {int r=(result); if (r<0) {gp_file_unref(file);return r;}}
+#define CL(result,list)  {int r=(result); if (r<0) {gp_list_free(list); return r;}}
 
 int
 delete_all_action (GPParams *p)
@@ -105,12 +87,14 @@ action_camera_upload_file (GPParams *p, const char *folder, const char *path)
 int
 num_files_action (GPParams *p)
 {
-	CameraList list;
+	CameraList *list;
 	int n;
 
-	CR (gp_camera_folder_list_files (p->camera, p->folder,
-					 &list, p->context));
-	CR (n = gp_list_count (&list));
+	CR (gp_list_new (&list));
+	CL (gp_camera_folder_list_files (p->camera, p->folder,
+					 list, p->context), list);
+	CL (n = gp_list_count (list), list);
+	gp_list_free (list);
 	if (p->quiet)
 		fprintf (stdout, "%i\n", n);
 	else
@@ -123,14 +107,16 @@ num_files_action (GPParams *p)
 int
 list_folders_action (GPParams *p)
 {
-	CameraList list;
+	CameraList *list;
 	int count;
 	const char *name;
 	unsigned int i;
+	
+	CR (gp_list_new (&list));
 
-	CR (gp_camera_folder_list_folders (p->camera, p->folder, &list,
-					   p->context));
-	CR (count = gp_list_count (&list));
+	CL (gp_camera_folder_list_folders (p->camera, p->folder, list,
+					   p->context), list);
+	CL (count = gp_list_count (list), list);
 	switch (count) {
         case 0:
                 printf (_("There are no folders in folder '%s'."), p->folder);
@@ -147,24 +133,25 @@ list_folders_action (GPParams *p)
                 break;
 	}
 	for (i = 0; i < count; i++) {
-		CR (gp_list_get_name (&list, i, &name));
+		CL (gp_list_get_name (list, i, &name), list);
 		printf (" - %s\n", name);
 	}
-
+	gp_list_free (list);
 	return (GP_OK);
 }
 
 int
 list_files_action (GPParams *p)
 {
-	CameraList list;
+	CameraList *list;
 	int count;
 	const char *name;
 	unsigned int i;
 
-	CR (gp_camera_folder_list_files (p->camera, p->folder, &list,
-					 p->context));
-	CR (count = gp_list_count (&list));
+	CR (gp_list_new (&list));
+	CL (gp_camera_folder_list_files (p->camera, p->folder, list,
+					 p->context), list);
+	CL (count = gp_list_count (list), list);
 	switch (count) {
 	case 0:
 		fprintf (stdout, _("There are no files in folder '%s'."),
@@ -183,10 +170,10 @@ list_files_action (GPParams *p)
 		break;
 	}
 	for (i = 0; i < count; i++) {
-		CR (gp_list_get_name (&list, i, &name));
-		CR (print_file_action (p, name));
+		CL (gp_list_get_name (list, i, &name), list);
+		CL (print_file_action (p, name), list);
 	}
-
+	gp_list_free (list);
 	return (GP_OK);
 }
 
@@ -334,6 +321,18 @@ save_audio_action (GPParams *p, const char *filename)
 				   GP_FILE_TYPE_AUDIO));
 }
 
+int
+save_all_audio_action (GPParams *p, const char *filename)
+{
+	/* not every file has an associated audio file */
+	if (camera_file_exists(p->camera, p->context, p->folder, filename,
+			       GP_FILE_TYPE_AUDIO))
+		return (save_file_to_file (p->camera, p->context, p->folder, filename,
+					   GP_FILE_TYPE_AUDIO));
+	else
+		return GP_OK;
+}
+
 int
 delete_file_action (GPParams *p, const char *filename)
 {
@@ -522,29 +521,30 @@ int
 auto_detect_action (GPParams *p)
 {
 	int x, count;
-        CameraList list;
+        CameraList *list;
         CameraAbilitiesList *al = NULL;
         GPPortInfoList *il = NULL;
         const char *name = NULL, *value = NULL;
 
+	CR (gp_list_new (&list));
         gp_abilities_list_new (&al);
         gp_abilities_list_load (al, p->context);
         gp_port_info_list_new (&il);
         gp_port_info_list_load (il);
-        gp_abilities_list_detect (al, il, &list, p->context);
+        gp_abilities_list_detect (al, il, list, p->context);
         gp_abilities_list_free (al);
         gp_port_info_list_free (il);
 
-        CR (count = gp_list_count (&list));
+        CL (count = gp_list_count (list), list);
 
         printf(_("%-30s %-16s\n"), _("Model"), _("Port"));
         printf(_("----------------------------------------------------------\n"));
         for (x = 0; x < count; x++) {
-                CR (gp_list_get_name  (&list, x, &name));
-                CR (gp_list_get_value (&list, x, &value));
+                CL (gp_list_get_name  (list, x, &name), list);
+                CL (gp_list_get_value (list, x, &value), list);
                 printf(_("%-30s %-16s\n"), name, value);
         }
-
+	gp_list_free (list);
         return GP_OK;
 }
 
@@ -553,6 +553,7 @@ action_camera_show_abilities (GPParams *p)
 {
 	CameraAbilities a;
 	int i;
+	int has_capture = 0;
 
 	CR (gp_camera_get_abilities (p->camera, &a));
 	printf (_("Abilities for camera             : %s\n"), a.model);
@@ -567,14 +568,25 @@ action_camera_show_abilities (GPParams *p)
 				a.speed[i]);
 	}
 	printf (_("Capture choices                  :\n"));
-	if (a.operations & GP_OPERATION_CAPTURE_IMAGE)
+	if (a.operations & GP_OPERATION_CAPTURE_IMAGE) {
 		printf (_("                                 : Image\n"));
-	if (a.operations & GP_OPERATION_CAPTURE_VIDEO)
+		has_capture = 1;
+	}
+	if (a.operations & GP_OPERATION_CAPTURE_VIDEO) {
 		printf (_("                                 : Video\n"));
-	if (a.operations & GP_OPERATION_CAPTURE_AUDIO)
+		has_capture = 1;
+	}
+	if (a.operations & GP_OPERATION_CAPTURE_AUDIO) {
 		printf (_("                                 : Audio\n"));
-	if (a.operations & GP_OPERATION_CAPTURE_PREVIEW)
+		has_capture = 1;
+	}
+	if (a.operations & GP_OPERATION_CAPTURE_PREVIEW) {
 		printf (_("                                 : Preview\n"));
+		has_capture = 1;
+	}
+	if (has_capture == 0) {
+		printf (_("                                 : Capture not supported by the driver\n"));
+	}
 	printf (_("Configuration support            : %s\n"),
 		(a.operations & GP_OPERATION_CONFIG) ? _("yes"):_("no"));
 	printf (_("Delete files on camera support   : %s\n"),
@@ -774,7 +786,7 @@ print_version_action (GPParams *p)
 	int n;
 	printf (_("gphoto2 %s\n"
 		  "\n"
-		  "Copyright (C) 2000-2002 Lutz Mueller and others\n"
+		  "Copyright (c) 2000-2003 Lutz Mueller and others\n"
 		  "%s"
 		  "\n"
 		  "gphoto2 comes with NO WARRANTY, to the extent permitted by law. You may\n"
@@ -882,12 +894,13 @@ debug_func (GPLogLevel level, const char *domain, const char *format,
 	    va_list args, void *data)
 {
 	struct timeval tv;
+	long sec, usec;
 
 	gettimeofday (&tv,NULL);
-	fprintf (stderr, "%li.%06li %s(%i): ",
-		 tv.tv_sec - glob_tv_zero.tv_sec,
-		 (1000000 + tv.tv_usec - glob_tv_zero.tv_usec) % 1000000,
-		 domain, level);
+	sec = tv.tv_sec  - glob_tv_zero.tv_sec;
+	usec = tv.tv_usec - glob_tv_zero.tv_usec;
+	if (usec < 0) {sec--; usec += 1000000L;}
+	fprintf (stderr, "%li.%06li %s(%i): ", sec, usec, domain, level);
 	vfprintf (stderr, format, args);
 	fprintf (stderr, "\n");
 }
@@ -922,10 +935,3 @@ debug_action (GPParams *p)
 
 	return (GP_OK);
 }
-
-/*
- * Local Variables:
- * c-file-style:"linux"
- * indent-tabs-mode:t
- * End:
- */
