@@ -1,6 +1,5 @@
-/* $Id: main.c,v 1.22 2002/11/06 21:24:14 lutz Exp $
- *
- * Copyright (C) 2002 Lutz Mller <lutz@users.sourceforge.net>
+/*
+ * Copyright  2002 Lutz Mller <lutz@users.sourceforge.net>
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
@@ -18,8 +17,22 @@
  * Boston, MA 02111-1307, USA.
  */
 
-#include <config.h>
+#include "config.h"
+#include "actions.h"
+#include "foreach.h"
+#include "gphoto2-port-info-list.h"
+#include "gphoto2-port-log.h"
+#include "gphoto2-setting.h"
+#include "gp-params.h"
+#include "i18n.h"
 #include "main.h"
+#include "options.h"
+#include "range.h"
+#include "shell.h"
+
+#ifdef HAVE_CDK
+#  include "gphoto2-cmd-config.h"
+#endif
 
 #include <stdlib.h>
 #include <stdio.h>
@@ -48,41 +61,6 @@
 #  include <signal.h>
 #endif
 
-#include "actions.h"
-#include "foreach.h"
-#include "options.h"
-#include "range.h"
-#include "shell.h"
-
-#ifdef HAVE_CDK
-#  include "gphoto2-cmd-config.h"
-#endif
-
-#include "gp-params.h"
-
-#include "gphoto2-port-info-list.h"
-#include "gphoto2-port-log.h"
-#include "gphoto2-setting.h"
-
-#ifdef ENABLE_NLS
-#  include <libintl.h>
-#  undef _
-#  define _(String) dgettext (PACKAGE, String)
-#  ifdef gettext_noop
-#    define N_(String) gettext_noop (String)
-#  else
-#    define N_(String) (String)
-#  endif 
-#else
-#  define textdomain(String) (String)
-#  define gettext(String) (String)
-#  define dgettext(Domain,Message) (Message)
-#  define dcgettext(Domain,Message,Type) (Message)
-#  define bindtextdomain(Domain,Directory) (Domain)
-#  define _(String) (String)
-#  define N_(String) (String)
-#endif
-
 #ifndef MAX
 # define MAX(a, b) ((a) > (b) ? (a) : (b))
 #endif
@@ -525,6 +503,7 @@ get_path_for_file (const char *folder, CameraFile *file, char **path)
 	CameraFileType type;
 	time_t t;
 	struct tm *tm;
+	int hour12;
 
 	if (!file || !path)
 		return (GP_ERROR_BAD_PARAMETERS);
@@ -533,6 +512,10 @@ get_path_for_file (const char *folder, CameraFile *file, char **path)
 	CR (gp_file_get_name (file, &name));
 	CR (gp_file_get_mtime (file, &t));
 	tm = localtime (&t);
+	hour12 = tm->tm_hour % 12;
+	if (hour12 == 0) {
+		hour12 = 12;
+	}
 
 	/*
 	 * If the user didn't specify a filename, use the original name 
@@ -631,19 +614,22 @@ get_path_for_file (const char *folder, CameraFile *file, char **path)
 				snprintf (b, sizeof (b), "%02i", tm->tm_mday);
 				break;
 			case 'H':
-			case 'k':
 				snprintf (b, sizeof (b), "%02i", tm->tm_hour);
 				break;
+			case 'k':
+				snprintf (b, sizeof (b), "%i", tm->tm_hour);
+				break;
 			case 'I':
+				snprintf (b, sizeof (b), "%02i", hour12);
+				break;
 			case 'l':
-				snprintf (b, sizeof (b), "%02i",
-					  tm->tm_hour % 12);
+				snprintf (b, sizeof (b), "%i", hour12);
 				break;
 			case 'j':
-				snprintf (b, sizeof (b), "%03i", tm->tm_yday);
+				snprintf (b, sizeof (b), "%03i", tm->tm_yday + 1);
 				break;
 			case 'm':
-				snprintf (b, sizeof (b), "%02i", tm->tm_mon);
+				snprintf (b, sizeof (b), "%02i", tm->tm_mon + 1);
 				break;
 			case 'M':
 				snprintf (b, sizeof (b), "%02i", tm->tm_min);
@@ -652,7 +638,7 @@ get_path_for_file (const char *folder, CameraFile *file, char **path)
 				snprintf (b, sizeof (b), "%02i", tm->tm_sec);
 				break;
 			case 'y':
-				snprintf (b, sizeof (b), "%02i", tm->tm_year);
+				snprintf (b, sizeof (b), "%02i", tm->tm_year % 100);
 				break;
 			case 'Y':
 				snprintf (b, sizeof (b), "%i",
@@ -738,12 +724,37 @@ save_camera_file_to_file (const char *folder, CameraFile *file)
 			s[strlen (s) - 1] = 0;
                 }
                 printf (_("Saving file as %s\n"), s);
+		fflush (stdout);
         }
 	CR (gp_file_save (file, s));
 
 	return (GP_OK);
 }
 
+int
+camera_file_exists (Camera *camera, GPContext *context, const char *folder,
+		    const char *filename, CameraFileType type)
+{
+	CameraFileInfo info;
+	CR (gp_camera_file_get_info (camera, folder, filename, &info, context));
+	switch (type) {
+	case GP_FILE_TYPE_AUDIO:
+		return(info.audio.fields != 0);
+		break;
+	case GP_FILE_TYPE_PREVIEW:
+		return(info.preview.fields != 0);
+		break;
+	case GP_FILE_TYPE_RAW:
+	case GP_FILE_TYPE_NORMAL:
+		return(info.file.fields != 0);
+		break;
+	default:
+		gp_context_error (context, "Unknown file type in camera_file_exists: %d", type);
+		return FALSE;
+		break;
+	}
+}
+
 int
 save_file_to_file (Camera *camera, GPContext *context, const char *folder,
 		   const char *filename, CameraFileType type)
@@ -862,7 +873,7 @@ OPTION_CALLBACK (get_audio_data)
 
 OPTION_CALLBACK (get_all_audio_data)
 {
-	CR (for_each_file (&p, save_audio_action));
+	CR (for_each_file (&p, save_all_audio_action));
 
 	return (GP_OK);
 }
@@ -1321,7 +1332,7 @@ cb_arg (poptContext ctx, enum poptCallbackReason reason,
 					delete_file_action, arg);
 		break;
 	case ARG_GET_ALL_AUDIO_DATA:
-		params->p.r = for_each_file (&p, save_audio_action);
+		params->p.r = for_each_file (&p, save_all_audio_action);
 		break;
 	case ARG_GET_ALL_FILES:
 		params->p.r = for_each_file (&p, save_file_action);
@@ -1359,7 +1370,7 @@ cb_arg (poptContext ctx, enum poptCallbackReason reason,
 	case ARG_MANUAL:
 		params->p.r = action_camera_manual (&p);
 		break;
-	case ARG_MKDIR:
+	case ARG_RMDIR:
 		params->p.r = gp_camera_folder_remove_dir (p.camera,
 				p.folder, arg, p.context);
 		break;
@@ -1369,7 +1380,7 @@ cb_arg (poptContext ctx, enum poptCallbackReason reason,
 		p.context = p.context;
 		params->p.r = num_files_action (&p);
 		break;
-	case ARG_RMDIR:
+	case ARG_MKDIR:
 		params->p.r = gp_camera_folder_make_dir (p.camera,
 				p.folder, arg, p.context);
 		break;
@@ -1430,7 +1441,7 @@ report_failure (int result, int argc, char **argv)
 			  "If you intend to send any error or "
 			  "debug messages to the gphoto\n"
 			  "developer mailing list "
-			  "<gphoto-devel@gphoto.org>, please run\n"
+			  "<gphoto-devel@lists.sourceforge.net>, please run\n"
 			  "gphoto2 as follows:\n\n"));
 
 		/*
@@ -1683,7 +1694,7 @@ main (int argc, char **argv)
 #endif
 		int count;
 		const char *model = NULL, *path = NULL;
-		CameraList list;
+		CameraList *list;
 		GPPortInfoList *il = NULL;
 		char buf[1024];
 
@@ -1692,14 +1703,15 @@ main (int argc, char **argv)
 
 		CR_MAIN (gp_port_info_list_new (&il));
 		CR_MAIN (gp_port_info_list_load (il));
+		CR_MAIN (gp_list_new (&list)); /* no freeing below */
 		CR_MAIN (gp_abilities_list_detect (p.abilities_list, il,
-						   &list, p.context));
-		CR_MAIN (count = gp_list_count (&list));
+						   list, p.context));
+		CR_MAIN (count = gp_list_count (list));
                 if (count == 1) {
 
                         /* Exactly one camera detected */
-			CR_MAIN (gp_list_get_name (&list, 0, &model));
-			CR_MAIN (gp_list_get_value (&list, 0, &path));
+			CR_MAIN (gp_list_get_name (list, 0, &model));
+			CR_MAIN (gp_list_get_value (list, 0, &path));
 			CR_MAIN (action_camera_set_model (&p, model));
 			CR_MAIN (action_camera_set_port (&p, path));
 
@@ -1718,13 +1730,14 @@ main (int argc, char **argv)
 
                         /* More than one camera detected */
                         /*FIXME: Let the user choose from the list!*/
-			CR_MAIN (gp_list_get_name (&list, 0, &model));
-			CR_MAIN (gp_list_get_value (&list, 0, &path));
+			CR_MAIN (gp_list_get_name (list, 0, &model));
+			CR_MAIN (gp_list_get_value (list, 0, &path));
 			CR_MAIN (action_camera_set_model (&p, model));
 			CR_MAIN (action_camera_set_port (&p, path));
                 }
 
 		CR_MAIN (gp_port_info_list_free (il));
+		gp_list_free (list);
         }
 
 	/*
