@@ -1,6 +1,6 @@
 /* main.h
  *
- * Copyright (C) 2002 Lutz Mller <lutz@users.sourceforge.net>
+ * Copyright  2002 Lutz Mller <lutz@users.sourceforge.net>
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
@@ -36,7 +36,7 @@ struct _Option {
         int     required;
 };
 
-#define MAX_IMAGE_NUMBER                1024
+#define MAX_IMAGE_NUMBER                16384
 
 #ifdef WIN32
 #include <io.h>
@@ -45,6 +45,9 @@ struct _Option {
 
 void 	cli_error_print(char *format, ...);
 
+int	camera_file_exists (Camera *camera, GPContext *context,
+			    const char *folder, const char *filename,
+			    CameraFileType type);
 int	save_file_to_file (Camera *camera, GPContext *context,
 			   const char *folder, const char *filename,
 			   CameraFileType type);
